#!groovy
import org.jfrog.stork.model.ModuleConfigContext
import org.jfrog.stork.model.module.JFrogModule

@Library('stork@infra/DEVF-858-pre77') _

static def sonarJobForBranch(String sourceBranch) {
    'Artifactory/artifactory-sonar-pipe/' + URLEncoder.encode(sourceBranch, "UTF-8")
}

storkConfig { ModuleConfigContext modCtx ->

    echo "Setting 'artifactory-oss' config"
    modCtx.moduleName = 'artifactory-oss'

    modCtx.init = { JFrogModule module ->
        module.allowVersionOverride('maven.plugin.surefire.version', 'RTFACT-15954')
        module.deployDocker = true
        module.dockerImageNames = ['jfrog/artifactory-oss']
        module.dockerRegistry = 'art.jfrog.io'
        module.dockerLocalRepo = 'art-docker-dev-local'
        module.setJDKName('adoptopenjdk-8u202')
    }

    modCtx.preBuild = { JFrogModule module ->
        echo "${module.name} extension env setup"
        module.settings.mailRecipients = module.settings.isRelease() ? 'stork@jfrog.com' : 'elig@jfrog.com'
        module.mvnProfiles = 'release,deb,rpm,docker'
        module.addMavenOption("-T 2C -DskipTests")

        if (!module.settings.isRelease()) {
            //Added this since we see some cases where classes are left in the target and mess up the build and we need them cleaned.
//            boolean cleanMvn = env.CLEAN_MVN ? env.CLEAN_MVN.toBoolean() : false
//            if (!cleanMvn) {
                //Default mvnCommand for the job is clean install
                module.setMavenPhases("clean install")
                module.addMavenOption("-Pdist-clean,reports-clean")
//            }
        }

        withCredentials([string(credentialsId: 'rpm-sign-passphrase', variable: 'SECRET')]) {
            wrap([$class: 'MaskPasswordsBuildWrapper', varPasswordPairs: [[password: SECRET, var: 'SECRET']]]) {
                module.settings.envVars << "passphrase=${SECRET}".toString()
            }
        }

        storkCommon.setUiBuildResource(module.settings)

        if (module.settings.isRelease() && !module.downstream) {
            // STRK-4 Update ArtifactoryVersion.java
            storkRelease.updateArtifactoryVersion(module)
        }

        sh "apt-get install -y unzip"

        // DEVF-684 - Disable automatic trigger of sonar due to jenkins issues
        /*Map<String, Closure> preTasks = [:]
        pushSonarJob(1, module, false, preTasks)
        parallel preTasks*/
    }

    modCtx.endOfBuild = { JFrogModule module ->
        if (module.settings.isRelease()) {
            echo "In release mode, no post build actions!"
            return
        }
        if (!module.deployArtifacts) {
            echo "Artifacts were not deployed to Artifactory, no post build actions!"
            return
        }
        echo "Post Build for ${module.name}"
        String ARTIFACTORY_TAG_VERSION = module.getDockerTagVersion()

        if (!module.isSuccessfulBuild()) {
            echo "Build was not successful, skipping tests!"
            return
        }

//        if (module.settings.isMultibranch() && !module.isMaster() && !BRANCH_NAME.endsWith("-rc")) {
//            // Just run sanity without waiting
//            Map<String, Closure> postTasks = [:]
//            postTasks['Trigger artifactory-sanity-5.x-branches'] = {
//                build job: 'Artifactory/artifactory-sanity-5.x-branches',
//                        parameters: [
//                                stringParam(name: 'artifactory_version', value: ARTIFACTORY_TAG_VERSION),
//                                stringParam(name: 'artifactory_type', value: 'oss'),
//                                stringParam(name: 'build_name', value: "${module.getDockerBuildInfoName()}")
//                        ], propagate: false, wait: false
//            }
//
//            parallel postTasks
//        } else {
//            // In non multibranch run strict tests if set
//            boolean strict = env.STRICT ? env.STRICT.toBoolean() : false
//            int nRetry = strict ? 2 : 1
//
//            Map<String, Closure> postTasks = [:]
//            postTasks['Trigger artifactory-sanity-5.x'] = {
//                retry(nRetry) {
//                    build job: 'Artifactory/artifactory-sanity-5.x',
//                            parameters: [
//                                    stringParam(name: 'artifactory_version', value: ARTIFACTORY_TAG_VERSION),
//                                    stringParam(name: 'artifactory_type', value: 'oss')
//                            ], propagate: strict, wait: strict
//
//                }
//            }
//            postTasks['artifactory-oss-5.x-db-unitest'] = {
//                retry(nRetry) {
//                    build job: 'Artifactory/artifactory-oss-5.x-db-unitest',
//                            parameters: [
//                                    booleanParam(name: 'RUN_TEST_DERBY', value: true),
//                                    booleanParam(name: 'RUN_TEST_MYSQL', value: true),
//                                    booleanParam(name: 'RUN_TEST_MSSQL', value: false), //mssql runs in its own job on windows os
//                                    booleanParam(name: 'RUN_TEST_ORACLE', value: true),
//                                    booleanParam(name: 'RUN_TEST_POSTGRESQL', value: true),
//                                    booleanParam(name: 'RUN_TEST_MARIADB', value: true),
//                                    stringParam(name: 'MVN_OVERWRITE_CMD',
//                                            value: '-U -f artifactory-oss/storage/db/pom.xml -Prelease clean test -fae ' +
//                                                    '-Dmaven.repo.local=/workspace/docker-builder/build/.repository ' +
//                                                    '-Dmaven.test.failure.ignore=true  -s ' +
//                                                    '/workspace/docker-builder/build/jenkins-settings.xml')
//                            ], propagate: strict, wait: strict
//                }
//            }
//            parallel postTasks
//        }
    }
}

private void pushSonarJob(int nRetry, JFrogModule module, boolean strict, Map<String, Closure> postTasks) {
    postTasks['Trigger Artifactory Sonar'] = {
        retry(nRetry) {
            build job: sonarJobForBranch(module.getSourceBranch() as String),
                    parameters: [
                            stringParam(name: 'BRANCH_NAME', value: module.getSourceBranch())
                    ], propagate: strict, wait: strict
        }
    }
}
