package org.artifactory.security;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.security.UserGroupService;
import org.artifactory.api.security.access.CreatedTokenInfo;
import org.artifactory.common.ArtifactoryHome;
import org.artifactory.common.ConstantValues;
import org.artifactory.security.access.AccessService;
import org.artifactory.security.props.auth.OauthManager;
import org.artifactory.security.props.auth.model.AuthenticationModel;
import org.artifactory.security.props.auth.model.OauthModel;
import org.artifactory.security.props.auth.model.TokenKeyValue;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.HashSet;
import java.util.Set;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

/**
 * @author Yuval Reches
 */
public class LoginHandlerImplTest extends ArtifactoryHomeBoundTest {

    private final String USERNAME = "test";
    private final String TOKEN = "token";

    @Mock
    private ArtifactoryContext artifactoryContextMock;
    @Mock
    private OauthManager oauthManager;
    @Mock
    private UserGroupService userGroupService;
    @Mock
    private AccessService accessService;

    private LoginHandlerImpl loginHandler;

    @BeforeMethod
    private void setup() {
        MockitoAnnotations.initMocks(this);
        loginHandler = Mockito.spy(new LoginHandlerImpl(oauthManager, userGroupService, accessService));
        ArtifactoryContextThreadBinder.bind(artifactoryContextMock);
    }

    @AfterMethod
    private void cleanup() {
        ArtifactoryHome.unbind();
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testDoBasicAuthWithDbWithExistingToken() {
        TokenKeyValue tokenKeyValue = getTokenKeyValue();
        when(oauthManager.getToken(USERNAME, true)).thenReturn(tokenKeyValue);
        OauthModel oauthModel = loginHandler.doBasicAuthWithDb(USERNAME);
        assertModel(oauthModel);
    }

    @Test
    public void testDoBasicAuthWithDbWithGeneratedToken() {
        mockUserInfo();
        mockCreatedTokenInfo();
        when(oauthManager.getToken(USERNAME)).thenReturn(null);
        OauthModel oauthModel = loginHandler.doBasicAuthWithDb(USERNAME);
        assertModel(oauthModel);
        Integer expiresIn = ((AuthenticationModel) oauthModel).getExpiresIn();
        Assert.assertEquals((int) expiresIn,
                ConstantValues.genericTokensCacheIdleTimeSecs.getInt());
    }

    @Test(expectedExceptions = RuntimeException.class, expectedExceptionsMessageRegExp = "failed to generate token " +
            "for authenticated user:.*")
    public void testDoBasicAuthWithDbWithException() {
        when(oauthManager.getToken(USERNAME)).thenReturn(null);
        loginHandler.doBasicAuthWithDb(USERNAME);
    }

    @Test
    public void testGenerateTokenNoGroups() {
        mockUserInfo();
        mockCreatedTokenInfo();
        TokenKeyValue tokenKeyValue = loginHandler.generateToken(TOKEN);
        Assert.assertNotNull(tokenKeyValue);
        Assert.assertEquals(tokenKeyValue.getToken(), TOKEN);
        Assert.assertEquals(tokenKeyValue.getKey(), "accesstoken");
    }

    private void mockCreatedTokenInfo() {
        CreatedTokenInfo createdTokenInfo = Mockito.mock(CreatedTokenInfo.class);
        when(createdTokenInfo.getTokenValue()).thenReturn(TOKEN);
        when(accessService.createToken(any())).thenReturn(createdTokenInfo);
    }

    private void mockUserInfo() {
        UserInfo userInfo = Mockito.mock(UserInfo.class);
        when(userGroupService.currentUser()).thenReturn(userInfo);
    }

    @Test
    public void testGetScope() {
        String memberOfGroups = "member-of-groups:";
        UserInfo userInfo = Mockito.mock(UserInfo.class);
        String scope = loginHandler.getScope(userInfo);
        Assert.assertEquals(scope, memberOfGroups + "*");
        // Check with groups
        Set<UserGroupInfo> groupInfos = new HashSet<>();
        createGroupInfo(groupInfos, "info1");
        createGroupInfo(groupInfos, "info2");
        createGroupInfo(groupInfos, "info3");

        when(userInfo.getGroups()).thenReturn(groupInfos);
        scope = loginHandler.getScope(userInfo);
        Assert.assertTrue(scope.contains(memberOfGroups));
        Assert.assertTrue(scope.contains("info1"));
        Assert.assertTrue(scope.contains("info2"));
        Assert.assertTrue(scope.contains("info3"));

        String groups = scope.substring(memberOfGroups.length());
        Assert.assertEquals(groups.split(",").length, 3);
    }

    private TokenKeyValue getTokenKeyValue() {
        return new TokenKeyValue("key", TOKEN);
    }

    private void assertModel(OauthModel oauthModel) {
        Assert.assertTrue(oauthModel instanceof AuthenticationModel);
        AuthenticationModel authenticationModel = (AuthenticationModel) oauthModel;
        Assert.assertEquals(authenticationModel.getToken(), TOKEN);
    }

    private void createGroupInfo(Set<UserGroupInfo> groupInfos, String groupName) {
        UserGroupInfo userGroupInfo = Mockito.mock(UserGroupInfo.class);
        when(userGroupInfo.toString()).thenReturn(groupName);
        groupInfos.add(userGroupInfo);
    }
}